@@ -36,6 +36,12 @@ impl DataFrame {
         multithreaded: bool,
         sorted: bool,
     ) -> Result<GroupBy> {
+        if by.is_empty() {
+            return Err(PolarsError::ComputeError(
+                "expected keys in groupby operation, got nothing".into(),
+            ));
+        }
+
         macro_rules! finish_packed_bit_path {
             ($ca0:expr, $ca1:expr, $pack_fn:expr) => {{
                 let n_partitions = set_partition_size();
@@ -144,7 +150,7 @@ impl DataFrame {
 
                     // arbitrarily chosen bound, if avg no of bytes to encode is larger than this
                     // value we fall back to default groupby
-                    if (lhs.get_values_size() + rhs.get_values_size()) / lhs.len() < 128 {
+                    if (lhs.get_values_size() + rhs.get_values_size()) / (lhs.len() + 1) < 128 {
                         pack_utf8_columns(lhs, rhs, n_partitions, sorted)
                     } else {
                         groupby_threaded_multiple_keys_flat(keys_df, n_partitions, sorted)
@@ -951,7 +957,7 @@ impl<'df> GroupBy<'df> {
     }
 }
 
-#[derive(Copy, Clone)]
+#[derive(Copy, Clone, Debug)]
 pub enum GroupByMethod {
     Min,
     Max,
