@@ -35,7 +35,6 @@ pub use crate::{
 };
 pub(crate) use arrow::array::*;
 pub use arrow::datatypes::{Field as ArrowField, Schema as ArrowSchema};
-pub use polars_arrow::prelude::{LargeListArray, LargeStringArray, QuantileInterpolOptions};
 pub(crate) use polars_arrow::trusted_len::TrustedLen;
 pub use std::sync::Arc;
 
@@ -56,6 +55,7 @@ pub use crate::chunked_array::ops::rolling_window::RollingOptions;
 pub use polars_arrow::kernels::ewm::EWMOptions;
 
 pub(crate) use polars_arrow::export::*;
+pub use polars_arrow::prelude::*;
 
 #[cfg(feature = "dtype-categorical")]
 pub use crate::chunked_array::logical::categorical::*;
