@@ -12,7 +12,7 @@ use crate::chunked_array::{
 };
 use crate::fmt::FmtList;
 use crate::frame::groupby::*;
-use crate::frame::hash_join::{HashJoin, ZipOuterJoinColumn};
+use crate::frame::hash_join::ZipOuterJoinColumn;
 use crate::prelude::*;
 use crate::series::implementations::SeriesWrap;
 use ahash::RandomState;
@@ -81,15 +81,7 @@ impl private::PrivateSeries for SeriesWrap<BooleanChunked> {
     fn agg_list(&self, groups: &GroupsProxy) -> Option<Series> {
         self.0.agg_list(groups)
     }
-    fn hash_join_inner(&self, other: &Series) -> Vec<(IdxSize, IdxSize)> {
-        HashJoin::hash_join_inner(&self.0, other.as_ref().as_ref())
-    }
-    fn hash_join_left(&self, other: &Series) -> Vec<(IdxSize, Option<IdxSize>)> {
-        HashJoin::hash_join_left(&self.0, other.as_ref().as_ref())
-    }
-    fn hash_join_outer(&self, other: &Series) -> Vec<(Option<IdxSize>, Option<IdxSize>)> {
-        HashJoin::hash_join_outer(&self.0, other.as_ref().as_ref())
-    }
+
     fn zip_outer_join_column(
         &self,
         right_column: &Series,
@@ -267,8 +259,8 @@ impl SeriesTrait for SeriesWrap<BooleanChunked> {
         ChunkSort::sort_with(&self.0, options).into_series()
     }
 
-    fn argsort(&self, reverse: bool) -> IdxCa {
-        ChunkSort::argsort(&self.0, reverse)
+    fn argsort(&self, options: SortOptions) -> IdxCa {
+        ChunkSort::argsort(&self.0, options)
     }
 
     fn null_count(&self) -> usize {
