@@ -12,7 +12,7 @@ use crate::chunked_array::{
 };
 use crate::fmt::FmtList;
 use crate::frame::groupby::*;
-use crate::frame::hash_join::{check_categorical_src, HashJoin, ZipOuterJoinColumn};
+use crate::frame::hash_join::{check_categorical_src, ZipOuterJoinColumn};
 use crate::prelude::*;
 use crate::series::implementations::SeriesWrap;
 use ahash::RandomState;
@@ -32,6 +32,7 @@ impl SeriesWrap<CategoricalChunked> {
         if keep_fast_unique && self.0.can_fast_unique() {
             out.set_fast_unique(true)
         }
+        out.set_lexical_sorted(self.0.use_lexical_sort());
         out
     }
 
@@ -106,21 +107,6 @@ impl private::PrivateSeries for SeriesWrap<CategoricalChunked> {
         })
     }
 
-    fn hash_join_inner(&self, other: &Series) -> Vec<(IdxSize, IdxSize)> {
-        self.0
-            .logical()
-            .hash_join_inner(other.categorical().unwrap().logical())
-    }
-    fn hash_join_left(&self, other: &Series) -> Vec<(IdxSize, Option<IdxSize>)> {
-        self.0
-            .logical()
-            .hash_join_left(other.categorical().unwrap().logical())
-    }
-    fn hash_join_outer(&self, other: &Series) -> Vec<(Option<IdxSize>, Option<IdxSize>)> {
-        self.0
-            .logical()
-            .hash_join_outer(other.categorical().unwrap().logical())
-    }
     fn zip_outer_join_column(
         &self,
         right_column: &Series,
@@ -302,10 +288,6 @@ impl SeriesTrait for SeriesWrap<CategoricalChunked> {
         self.0.cast(data_type)
     }
 
-    fn value_counts(&self) -> Result<DataFrame> {
-        self.0.value_counts()
-    }
-
     fn get(&self, index: usize) -> AnyValue {
         self.0.get_any_value(index)
     }
@@ -320,8 +302,8 @@ impl SeriesTrait for SeriesWrap<CategoricalChunked> {
         self.0.sort_with(options).into_series()
     }
 
-    fn argsort(&self, reverse: bool) -> IdxCa {
-        self.0.argsort(reverse)
+    fn argsort(&self, options: SortOptions) -> IdxCa {
+        self.0.argsort(options)
     }
 
     fn null_count(&self) -> usize {
