@@ -57,12 +57,6 @@ macro_rules! impl_dyn_series {
                 self.0.cummin(reverse).$into_logical().into_series()
             }
 
-            #[cfg(feature = "asof_join")]
-            fn join_asof(&self, other: &Series) -> Result<Vec<Option<IdxSize>>> {
-                let other = other.to_physical_repr();
-                self.0.deref().join_asof(&other)
-            }
-
             fn set_sorted(&mut self, reverse: bool) {
                 self.0.deref_mut().set_sorted(reverse)
             }
@@ -140,18 +134,6 @@ macro_rules! impl_dyn_series {
                     .map(|s| s.$into_logical().into_series())
             }
 
-            fn hash_join_inner(&self, other: &Series) -> Vec<(IdxSize, IdxSize)> {
-                let other = other.to_physical_repr().into_owned();
-                self.0.hash_join_inner(&other.as_ref().as_ref())
-            }
-            fn hash_join_left(&self, other: &Series) -> Vec<(IdxSize, Option<IdxSize>)> {
-                let other = other.to_physical_repr().into_owned();
-                self.0.hash_join_left(&other.as_ref().as_ref())
-            }
-            fn hash_join_outer(&self, other: &Series) -> Vec<(Option<IdxSize>, Option<IdxSize>)> {
-                let other = other.to_physical_repr().into_owned();
-                self.0.hash_join_outer(&other.as_ref().as_ref())
-            }
             fn zip_outer_join_column(
                 &self,
                 right_column: &Series,
@@ -397,10 +379,6 @@ macro_rules! impl_dyn_series {
                 self.0.to_dummies()
             }
 
-            fn value_counts(&self) -> Result<DataFrame> {
-                self.0.value_counts()
-            }
-
             fn get(&self, index: usize) -> AnyValue {
                 self.0.get_any_value(index)
             }
@@ -415,8 +393,8 @@ macro_rules! impl_dyn_series {
                 self.0.sort_with(options).$into_logical().into_series()
             }
 
-            fn argsort(&self, reverse: bool) -> IdxCa {
-                self.0.argsort(reverse)
+            fn argsort(&self, options: SortOptions) -> IdxCa {
+                self.0.argsort(options)
             }
 
             fn null_count(&self) -> usize {
@@ -670,7 +648,7 @@ mod test {
     }
 
     #[test]
-    #[cfg(feature = "dtype-datetime")]
+    #[cfg(all(feature = "dtype-datetime", feature = "dtype-duration"))]
     fn test_datelike_methods() -> Result<()> {
         let s = Series::new("foo", &[1, 2, 3]);
         let s = s.cast(&DataType::Datetime(TimeUnit::Nanoseconds, None))?;
@@ -689,7 +667,7 @@ mod test {
     }
 
     #[test]
-    #[cfg(feature = "dtype-datetime")]
+    #[cfg(all(feature = "dtype-datetime", feature = "dtype-duration"))]
     fn test_arithmetic_dispatch() {
         let s = Int64Chunked::new("", &[1, 2, 3])
             .into_datetime(TimeUnit::Nanoseconds, None)
