@@ -59,12 +59,6 @@ impl private::PrivateSeries for SeriesWrap<DurationChunked> {
             .into_series()
     }
 
-    #[cfg(feature = "asof_join")]
-    fn join_asof(&self, other: &Series) -> Result<Vec<Option<IdxSize>>> {
-        let other = other.to_physical_repr();
-        self.0.deref().join_asof(&other)
-    }
-
     fn set_sorted(&mut self, reverse: bool) {
         self.0.deref_mut().set_sorted(reverse)
     }
@@ -146,18 +140,6 @@ impl private::PrivateSeries for SeriesWrap<DurationChunked> {
             .map(|s| s.into_duration(self.0.time_unit()).into_series())
     }
 
-    fn hash_join_inner(&self, other: &Series) -> Vec<(IdxSize, IdxSize)> {
-        let other = other.to_physical_repr().into_owned();
-        self.0.hash_join_inner(other.as_ref().as_ref())
-    }
-    fn hash_join_left(&self, other: &Series) -> Vec<(IdxSize, Option<IdxSize>)> {
-        let other = other.to_physical_repr().into_owned();
-        self.0.hash_join_left(other.as_ref().as_ref())
-    }
-    fn hash_join_outer(&self, other: &Series) -> Vec<(Option<IdxSize>, Option<IdxSize>)> {
-        let other = other.to_physical_repr().into_owned();
-        self.0.hash_join_outer(other.as_ref().as_ref())
-    }
     fn zip_outer_join_column(
         &self,
         right_column: &Series,
@@ -394,10 +376,6 @@ impl SeriesTrait for SeriesWrap<DurationChunked> {
         self.0.to_dummies()
     }
 
-    fn value_counts(&self) -> Result<DataFrame> {
-        self.0.value_counts()
-    }
-
     fn get(&self, index: usize) -> AnyValue {
         self.0.get_any_value(index)
     }
@@ -417,8 +395,8 @@ impl SeriesTrait for SeriesWrap<DurationChunked> {
             .into_series()
     }
 
-    fn argsort(&self, reverse: bool) -> IdxCa {
-        self.0.argsort(reverse)
+    fn argsort(&self, options: SortOptions) -> IdxCa {
+        self.0.argsort(options)
     }
 
     fn null_count(&self) -> usize {
