@@ -1518,7 +1518,7 @@ impl Expr {
     pub fn dot(self, other: Expr) -> Expr {
         let function = |s: &mut [Series]| Ok((&s[0] * &s[1]).sum_as_series());
 
-        self.map_many(function, &[other], GetOutput::same_type())
+        self.apply_many(function, &[other], GetOutput::same_type())
             .with_fmt("dot")
     }
 
@@ -2141,7 +2141,11 @@ impl Expr {
                 }
             }),
         )
-        .with_fmt("entropy")
+        .with_function_options(|mut options| {
+            options.fmt_str = "entropy";
+            options.auto_explode = true;
+            options
+        })
     }
 
     #[cfg(feature = "strings")]
