@@ -212,16 +212,12 @@ impl LogicalPlanBuilder {
     pub fn project_local(self, exprs: Vec<Expr>) -> Self {
         let (exprs, schema) =
             try_delayed!(prepare_projection(exprs, self.0.schema()), &self.0, into);
-        if !exprs.is_empty() {
-            LogicalPlan::LocalProjection {
-                expr: exprs,
-                input: Box::new(self.0),
-                schema: Arc::new(schema),
-            }
-            .into()
-        } else {
-            self
+        LogicalPlan::LocalProjection {
+            expr: exprs,
+            input: Box::new(self.0),
+            schema: Arc::new(schema),
         }
+        .into()
     }
 
     pub fn fill_null(self, fill_value: Expr) -> Self {
@@ -313,6 +309,25 @@ impl LogicalPlanBuilder {
         );
         schema.merge(other);
 
+        let index_columns = &[
+            rolling_options
+                .as_ref()
+                .map(|options| &options.index_column),
+            dynamic_options
+                .as_ref()
+                .map(|options| &options.index_column),
+        ];
+        for &name in index_columns.iter().flatten() {
+            let dtype = try_delayed!(
+                current_schema
+                    .get(name)
+                    .ok_or_else(|| PolarsError::NotFound(name.clone())),
+                self.0,
+                into
+            );
+            schema.with_column(name.clone(), dtype.clone());
+        }
+
         LogicalPlan::Aggregate {
             input: Box::new(self.0),
             keys,
@@ -357,11 +372,19 @@ impl LogicalPlanBuilder {
 
     pub fn explode(self, columns: Vec<Expr>) -> Self {
         let columns = rewrite_projections(columns, self.0.schema(), &[]);
+
+        let mut schema = (**self.0.schema()).clone();
+
         // columns to string
         let columns = columns
             .iter()
             .map(|e| {
                 if let Expr::Column(name) = e {
+                    if let Some(DataType::List(inner)) = schema.get(name) {
+                        let inner = *inner.clone();
+                        schema.with_column(name.to_string(), inner)
+                    }
+
                     (**name).to_owned()
                 } else {
                     panic!("expected column expression")
@@ -371,6 +394,7 @@ impl LogicalPlanBuilder {
         LogicalPlan::Explode {
             input: Box::new(self.0),
             columns,
+            schema: Arc::new(schema),
         }
         .into()
     }
