@@ -289,9 +289,17 @@ pub(crate) fn to_alp(
                 args,
             }
         }
-        LogicalPlan::Explode { input, columns } => {
+        LogicalPlan::Explode {
+            input,
+            columns,
+            schema,
+        } => {
             let input = to_alp(*input, expr_arena, lp_arena)?;
-            ALogicalPlan::Explode { input, columns }
+            ALogicalPlan::Explode {
+                input,
+                columns,
+                schema,
+            }
         }
         LogicalPlan::Cache { input } => {
             let input = to_alp(*input, expr_arena, lp_arena)?;
@@ -739,9 +747,17 @@ pub(crate) fn node_to_lp(
                 args,
             }
         }
-        ALogicalPlan::Explode { input, columns } => {
+        ALogicalPlan::Explode {
+            input,
+            columns,
+            schema,
+        } => {
             let input = Box::new(node_to_lp(input, expr_arena, lp_arena));
-            LogicalPlan::Explode { input, columns }
+            LogicalPlan::Explode {
+                input,
+                columns,
+                schema,
+            }
         }
         ALogicalPlan::Cache { input } => {
             let input = Box::new(node_to_lp(input, expr_arena, lp_arena));
