@@ -8,12 +8,14 @@ pub struct StackExec {
     pub(crate) input: Box<dyn Executor>,
     pub(crate) has_windows: bool,
     pub(crate) expr: Vec<Arc<dyn PhysicalExpr>>,
+    pub(crate) input_schema: SchemaRef,
 }
 
 impl Executor for StackExec {
     fn execute(&mut self, state: &ExecutionState) -> Result<DataFrame> {
         let mut df = self.input.execute(state)?;
 
+        state.set_schema(self.input_schema.clone());
         let res = if self.has_windows {
             // we have a different run here
             // to ensure the window functions run sequential and share caches
@@ -26,12 +28,14 @@ impl Executor for StackExec {
                     .collect::<Result<Vec<_>>>()
             })?
         };
+        state.clear_schema_cache();
+        state.clear_expr_cache();
 
+        let schema = &*self.input_schema;
         for s in res {
-            df.with_column(s)?;
+            df.with_column_and_schema(s, schema)?;
         }
 
-        state.clear_expr_cache();
         Ok(df)
     }
 }
