@@ -276,7 +276,7 @@ impl DefaultPlanner {
                     args,
                 }))
             }
-            Explode { input, columns } => {
+            Explode { input, columns, .. } => {
                 let input = self.create_physical_plan(input, lp_arena, expr_arena)?;
                 Ok(Box::new(ExplodeExec { input, columns }))
             }
@@ -331,6 +331,7 @@ impl DefaultPlanner {
                 if let Some(options) = options.rolling {
                     return Ok(Box::new(GroupByRollingExec {
                         input,
+                        keys: phys_keys,
                         aggs: phys_aggs,
                         options,
                         input_schema,
