@@ -5,7 +5,7 @@ use crate::Wrap;
 use polars::chunked_array::builder::get_list_builder;
 use polars::prelude::*;
 use pyo3::prelude::*;
-use pyo3::types::{PyBool, PyCFunction, PyDict, PyFloat, PyInt, PyList, PyString, PyTuple};
+use pyo3::types::{PyBool, PyCFunction, PyDict, PyFloat, PyList, PyString, PyTuple};
 
 /// Find the output type and dispatch to that implementation.
 fn infer_and_finish<'a, A: ApplyLambda<'a>>(
@@ -15,12 +15,12 @@ fn infer_and_finish<'a, A: ApplyLambda<'a>>(
     out: &'a PyAny,
     null_count: usize,
 ) -> PyResult<PySeries> {
-    if out.is_instance::<PyBool>().unwrap() {
+    if out.is_instance_of::<PyBool>().unwrap() {
         let first_value = out.extract::<bool>().unwrap();
         applyer
             .apply_lambda_with_bool_out_type(py, lambda, null_count, Some(first_value))
             .map(|ca| ca.into_series().into())
-    } else if out.is_instance::<PyFloat>().unwrap() {
+    } else if out.is_instance_of::<PyFloat>().unwrap() {
         let first_value = out.extract::<f64>().unwrap();
         applyer
             .apply_lambda_with_primitive_out_type::<Float64Type>(
@@ -30,17 +30,7 @@ fn infer_and_finish<'a, A: ApplyLambda<'a>>(
                 Some(first_value),
             )
             .map(|ca| ca.into_series().into())
-    } else if out.is_instance::<PyInt>().unwrap() {
-        let first_value = out.extract::<i64>().unwrap();
-        applyer
-            .apply_lambda_with_primitive_out_type::<Int64Type>(
-                py,
-                lambda,
-                null_count,
-                Some(first_value),
-            )
-            .map(|ca| ca.into_series().into())
-    } else if out.is_instance::<PyString>().unwrap() {
+    } else if out.is_instance_of::<PyString>().unwrap() {
         let first_value = out.extract::<&str>().unwrap();
         applyer
             .apply_lambda_with_utf8_out_type(py, lambda, null_count, Some(first_value))
@@ -52,7 +42,7 @@ fn infer_and_finish<'a, A: ApplyLambda<'a>>(
         applyer
             .apply_lambda_with_list_out_type(py, lambda.to_object(py), null_count, &series, dt)
             .map(|ca| ca.into_series().into())
-    } else if out.is_instance::<PyList>().unwrap() {
+    } else if out.is_instance_of::<PyList>().unwrap() {
         let pypolars = PyModule::import(py, "polars").unwrap().to_object(py);
         let series = pypolars.getattr(py, "Series").unwrap().call1(py, (out,))?;
         let py_pyseries = series.getattr(py, "_s").unwrap();
@@ -77,9 +67,26 @@ fn infer_and_finish<'a, A: ApplyLambda<'a>>(
         applyer
             .apply_lambda_with_list_out_type(py, new_lambda, null_count, &series, dt)
             .map(|ca| ca.into_series().into())
-    } else if out.is_instance::<PyTuple>().unwrap() {
+    } else if out.is_instance_of::<PyTuple>().unwrap() {
         let first = out.extract::<Wrap<AnyValue<'_>>>()?;
         applyer.apply_to_struct(py, lambda, null_count, first.0)
+    }
+    // this succeeds for numpy ints as well, where checking if it is pyint fails
+    // we do this later in the chain so that we don't extract integers from string chars.
+    else if out.extract::<i64>().is_ok() {
+        let first_value = out.extract::<i64>().unwrap();
+        applyer
+            .apply_lambda_with_primitive_out_type::<Int64Type>(
+                py,
+                lambda,
+                null_count,
+                Some(first_value),
+            )
+            .map(|ca| ca.into_series().into())
+    } else if let Ok(av) = out.extract::<Wrap<AnyValue>>() {
+        applyer
+            .apply_extract_any_values(py, lambda, null_count, av.0)
+            .map(|s| s.into())
     } else {
         applyer
             .apply_lambda_with_object_out_type(
@@ -147,6 +154,14 @@ pub trait ApplyLambda<'a> {
         dt: &DataType,
     ) -> PyResult<ListChunked>;
 
+    fn apply_extract_any_values(
+        &'a self,
+        py: Python,
+        lambda: &'a PyAny,
+        init_null_count: usize,
+        first_value: AnyValue<'a>,
+    ) -> PyResult<Series>;
+
     /// Apply a lambda with list output type
     fn apply_lambda_with_object_out_type(
         &'a self,
@@ -401,6 +416,40 @@ impl<'a> ApplyLambda<'a> for BooleanChunked {
         }
     }
 
+    fn apply_extract_any_values(
+        &'a self,
+        py: Python,
+        lambda: &'a PyAny,
+        init_null_count: usize,
+        first_value: AnyValue<'a>,
+    ) -> PyResult<Series> {
+        let mut avs = Vec::with_capacity(self.len());
+        avs.extend(std::iter::repeat(AnyValue::Null).take(init_null_count));
+        avs.push(first_value);
+
+        if self.null_count() > 0 {
+            let iter = self.into_iter().skip(init_null_count + 1).map(|opt_val| {
+                let out_wrapped = match opt_val {
+                    None => Wrap(AnyValue::Null),
+                    Some(val) => call_lambda_and_extract(py, lambda, val).unwrap(),
+                };
+                out_wrapped.0
+            });
+            avs.extend(iter);
+        } else {
+            let iter = self
+                .into_no_null_iter()
+                .skip(init_null_count + 1)
+                .map(|val| {
+                    call_lambda_and_extract::<_, Wrap<AnyValue>>(py, lambda, val)
+                        .unwrap()
+                        .0
+                });
+            avs.extend(iter);
+        }
+        Ok(Series::new(self.name(), &avs))
+    }
+
     fn apply_lambda_with_object_out_type(
         &'a self,
         py: Python,
@@ -662,6 +711,40 @@ where
         }
     }
 
+    fn apply_extract_any_values(
+        &'a self,
+        py: Python,
+        lambda: &'a PyAny,
+        init_null_count: usize,
+        first_value: AnyValue<'a>,
+    ) -> PyResult<Series> {
+        let mut avs = Vec::with_capacity(self.len());
+        avs.extend(std::iter::repeat(AnyValue::Null).take(init_null_count));
+        avs.push(first_value);
+
+        if self.null_count() > 0 {
+            let iter = self.into_iter().skip(init_null_count + 1).map(|opt_val| {
+                let out_wrapped = match opt_val {
+                    None => Wrap(AnyValue::Null),
+                    Some(val) => call_lambda_and_extract(py, lambda, val).unwrap(),
+                };
+                out_wrapped.0
+            });
+            avs.extend(iter);
+        } else {
+            let iter = self
+                .into_no_null_iter()
+                .skip(init_null_count + 1)
+                .map(|val| {
+                    call_lambda_and_extract::<_, Wrap<AnyValue>>(py, lambda, val)
+                        .unwrap()
+                        .0
+                });
+            avs.extend(iter);
+        }
+        Ok(Series::new(self.name(), &avs))
+    }
+
     fn apply_lambda_with_object_out_type(
         &'a self,
         py: Python,
@@ -917,6 +1000,40 @@ impl<'a> ApplyLambda<'a> for Utf8Chunked {
         }
     }
 
+    fn apply_extract_any_values(
+        &'a self,
+        py: Python,
+        lambda: &'a PyAny,
+        init_null_count: usize,
+        first_value: AnyValue<'a>,
+    ) -> PyResult<Series> {
+        let mut avs = Vec::with_capacity(self.len());
+        avs.extend(std::iter::repeat(AnyValue::Null).take(init_null_count));
+        avs.push(first_value);
+
+        if self.null_count() > 0 {
+            let iter = self.into_iter().skip(init_null_count + 1).map(|opt_val| {
+                let out_wrapped = match opt_val {
+                    None => Wrap(AnyValue::Null),
+                    Some(val) => call_lambda_and_extract(py, lambda, val).unwrap(),
+                };
+                out_wrapped.0
+            });
+            avs.extend(iter);
+        } else {
+            let iter = self
+                .into_no_null_iter()
+                .skip(init_null_count + 1)
+                .map(|val| {
+                    call_lambda_and_extract::<_, Wrap<AnyValue>>(py, lambda, val)
+                        .unwrap()
+                        .0
+                });
+            avs.extend(iter);
+        }
+        Ok(Series::new(self.name(), &avs))
+    }
+
     fn apply_lambda_with_object_out_type(
         &'a self,
         py: Python,
@@ -1380,6 +1497,52 @@ impl<'a> ApplyLambda<'a> for ListChunked {
             ))
         }
     }
+
+    fn apply_extract_any_values(
+        &'a self,
+        py: Python,
+        lambda: &'a PyAny,
+        init_null_count: usize,
+        first_value: AnyValue<'a>,
+    ) -> PyResult<Series> {
+        let pypolars = PyModule::import(py, "polars")?;
+        let mut avs = Vec::with_capacity(self.len());
+        avs.extend(std::iter::repeat(AnyValue::Null).take(init_null_count));
+        avs.push(first_value);
+
+        let call_with_value = |val: Series| {
+            // create a PySeries struct/object for Python
+            let pyseries = PySeries::new(val);
+            // Wrap this PySeries object in the python side Series wrapper
+            let python_series_wrapper = pypolars
+                .getattr("wrap_s")
+                .unwrap()
+                .call1((pyseries,))
+                .unwrap();
+            call_lambda_and_extract::<_, Wrap<AnyValue>>(py, lambda, python_series_wrapper)
+                .unwrap()
+                .0
+        };
+
+        if self.null_count() > 0 {
+            let iter = self
+                .into_iter()
+                .skip(init_null_count + 1)
+                .map(|opt_val| match opt_val {
+                    None => AnyValue::Null,
+                    Some(val) => call_with_value(val),
+                });
+            avs.extend(iter);
+        } else {
+            let iter = self
+                .into_no_null_iter()
+                .skip(init_null_count + 1)
+                .map(|val| call_with_value(val));
+            avs.extend(iter);
+        }
+        Ok(Series::new(self.name(), &avs))
+    }
+
     fn apply_lambda_with_object_out_type(
         &'a self,
         py: Python,
@@ -1644,6 +1807,40 @@ impl<'a> ApplyLambda<'a> for ObjectChunked<ObjectValue> {
         }
     }
 
+    fn apply_extract_any_values(
+        &'a self,
+        py: Python,
+        lambda: &'a PyAny,
+        init_null_count: usize,
+        first_value: AnyValue<'a>,
+    ) -> PyResult<Series> {
+        let mut avs = Vec::with_capacity(self.len());
+        avs.extend(std::iter::repeat(AnyValue::Null).take(init_null_count));
+        avs.push(first_value);
+
+        if self.null_count() > 0 {
+            let iter = self.into_iter().skip(init_null_count + 1).map(|opt_val| {
+                let out_wrapped = match opt_val {
+                    None => Wrap(AnyValue::Null),
+                    Some(val) => call_lambda_and_extract(py, lambda, val).unwrap(),
+                };
+                out_wrapped.0
+            });
+            avs.extend(iter);
+        } else {
+            let iter = self
+                .into_no_null_iter()
+                .skip(init_null_count + 1)
+                .map(|val| {
+                    call_lambda_and_extract::<_, Wrap<AnyValue>>(py, lambda, val)
+                        .unwrap()
+                        .0
+                });
+            avs.extend(iter);
+        }
+        Ok(Series::new(self.name(), &avs))
+    }
+
     fn apply_lambda_with_object_out_type(
         &'a self,
         py: Python,
@@ -1834,6 +2031,30 @@ impl<'a> ApplyLambda<'a> for StructChunked {
             self.len(),
         ))
     }
+
+    fn apply_extract_any_values(
+        &'a self,
+        py: Python,
+        lambda: &'a PyAny,
+        init_null_count: usize,
+        first_value: AnyValue<'a>,
+    ) -> PyResult<Series> {
+        let names = self.fields().iter().map(|s| s.name()).collect::<Vec<_>>();
+        let mut avs = Vec::with_capacity(self.len());
+        avs.extend(std::iter::repeat(AnyValue::Null).take(init_null_count));
+        avs.push(first_value);
+
+        let iter = self.into_iter().skip(init_null_count + 1).map(|val| {
+            let arg = make_dict_arg(py, &names, val);
+            call_lambda_and_extract::<_, Wrap<AnyValue>>(py, lambda, arg)
+                .unwrap()
+                .0
+        });
+        avs.extend(iter);
+
+        Ok(Series::new(self.name(), &avs))
+    }
+
     fn apply_lambda_with_object_out_type(
         &'a self,
         py: Python,
