@@ -1,6 +1,8 @@
 from functools import reduce
 from typing import List, Optional
 
+import numpy as np
+
 import polars as pl
 
 
@@ -107,3 +109,37 @@ def test_apply_struct() -> None:
     )
 
     assert out.frame_equal(expected)
+
+
+def test_apply_numpy_out_3057() -> None:
+    df = pl.DataFrame(
+        dict(
+            id=[0, 0, 0, 1, 1, 1], t=[2.0, 4.3, 5, 10, 11, 14], y=[0.0, 1, 1.3, 2, 3, 4]
+        )
+    )
+
+    assert (
+        df.groupby("id", maintain_order=True)
+        .agg(
+            pl.apply(["y", "t"], lambda lst: np.trapz(y=lst[0], x=lst[1])).alias(
+                "result"
+            )
+        )
+        .frame_equal(pl.DataFrame({"id": [0, 1], "result": [1.955, 13.0]}))
+    )
+
+
+def test_apply_numpy_int_out() -> None:
+    df = pl.DataFrame({"col1": [2, 4, 8, 16]})
+    assert df.with_column(
+        pl.col("col1").apply(lambda x: np.left_shift(x, 8)).alias("result")
+    ).frame_equal(
+        pl.DataFrame({"col1": [2, 4, 8, 16], "result": [512, 1024, 2048, 4096]})
+    )
+    df = pl.DataFrame({"col1": [2, 4, 8, 16], "shift": [1, 1, 2, 2]})
+
+    assert df.select(
+        pl.struct(["col1", "shift"])
+        .apply(lambda cols: np.left_shift(cols["col1"], cols["shift"]))
+        .alias("result")
+    ).frame_equal(pl.DataFrame({"result": [4, 8, 32, 64]}))
