@@ -1216,7 +1216,18 @@ def test_hashing_on_python_objects() -> None:
     # this requires that the hashing and aggregations are done on python objects
 
     df = pl.DataFrame({"a": [1, 1, 3, 4], "b": [1, 1, 2, 2]})
-    df = df.with_column(pl.col("a").apply(lambda x: datetime(2021, 1, 1)).alias("foo"))
+
+    class Foo:
+        def __init__(self):  # type: ignore
+            pass
+
+        def __hash__(self):  # type: ignore
+            return 0
+
+        def __eq__(self, other):  # type: ignore
+            return True
+
+    df = df.with_column(pl.col("a").apply(lambda x: Foo()).alias("foo"))
     assert df.groupby(["foo"]).first().shape == (1, 3)
     assert df.distinct().shape == (3, 3)
 
@@ -1319,97 +1330,6 @@ Series: 'cat_column' [list]
     )
 
 
-def test_asof_join() -> None:
-    fmt = "%F %T%.3f"
-    dates = """2016-05-25 13:30:00.023
-2016-05-25 13:30:00.023
-2016-05-25 13:30:00.030
-2016-05-25 13:30:00.041
-2016-05-25 13:30:00.048
-2016-05-25 13:30:00.049
-2016-05-25 13:30:00.072
-2016-05-25 13:30:00.075""".split(
-        "\n"
-    )
-
-    ticker = """GOOG
-MSFT
-MSFT
-MSFT
-GOOG
-AAPL
-GOOG
-MSFT""".split(
-        "\n"
-    )
-
-    quotes = pl.DataFrame(
-        {
-            "dates": pl.Series(dates).str.strptime(pl.Datetime, fmt=fmt),
-            "ticker": ticker,
-            "bid": [720.5, 51.95, 51.97, 51.99, 720.50, 97.99, 720.50, 52.01],
-        }
-    )
-
-    dates = """2016-05-25 13:30:00.023
-2016-05-25 13:30:00.038
-2016-05-25 13:30:00.048
-2016-05-25 13:30:00.048
-2016-05-25 13:30:00.048""".split(
-        "\n"
-    )
-
-    ticker = """MSFT
-MSFT
-GOOG
-GOOG
-AAPL""".split(
-        "\n"
-    )
-
-    trades = pl.DataFrame(
-        {
-            "dates": pl.Series(dates).str.strptime(pl.Datetime, fmt=fmt),
-            "ticker": ticker,
-            "bid": [51.95, 51.95, 720.77, 720.92, 98.0],
-        }
-    )
-
-    out = trades.join_asof(quotes, on="dates", strategy="backward")
-    assert out.columns == ["dates", "ticker", "bid", "ticker_right", "bid_right"]
-    assert (out["dates"].cast(int) / 1000).to_list() == [
-        1464183000023,
-        1464183000038,
-        1464183000048,
-        1464183000048,
-        1464183000048,
-    ]
-    assert trades.join_asof(quotes, on="dates", strategy="forward")[
-        "bid_right"
-    ].to_list() == [720.5, 51.99, 720.5, 720.5, 720.5]
-
-    out = trades.join_asof(quotes, on="dates", by="ticker")
-    assert out["bid_right"].to_list() == [51.95, 51.97, 720.5, 720.5, None]
-
-    out = quotes.join_asof(trades, on="dates", by="ticker")
-    assert out["bid_right"].to_list() == [
-        None,
-        51.95,
-        51.95,
-        51.95,
-        720.92,
-        98.0,
-        720.92,
-        51.95,
-    ]
-    assert quotes.join_asof(trades, on="dates", strategy="backward", tolerance="5ms")[
-        "bid_right"
-    ].to_list() == [51.95, 51.95, None, 51.95, 98.0, 98.0, None, None]
-    assert quotes.join_asof(trades, on="dates", strategy="forward", tolerance="5ms")[
-        "bid_right"
-    ].to_list() == [51.95, 51.95, None, None, 720.77, None, None, None]
-
-
 def test_groupby_agg_n_unique_floats() -> None:
     # tests proper dispatch
     df = pl.DataFrame({"a": [1, 1, 3], "b": [1.0, 2.0, 2.0]})
@@ -1950,3 +1870,32 @@ def test_explode_empty() -> None:
         .agg(pl.col("y").take([]))
     )
     assert df.explode("y").shape == (0, 2)
+
+    df = pl.DataFrame(dict(x=["1", "2", "4"], y=[["a", "b", "c"], ["d"], []]))
+    assert df.explode("y").frame_equal(
+        pl.DataFrame({"x": ["1", "1", "1", "2"], "y": ["a", "b", "c", "d"]})
+    )
+
+
+def test_asof_by_multiple_keys() -> None:
+    lhs = pl.DataFrame(
+        {
+            "a": [-20, -19, 8, 12, 14],
+            "by": [1, 1, 2, 2, 2],
+            "by2": [1, 1, 2, 2, 2],
+        }
+    )
+
+    rhs = pl.DataFrame(
+        {
+            "a": [-19, -15, 3, 5, 13],
+            "by": [1, 1, 2, 2, 2],
+            "by2": [1, 1, 2, 2, 2],
+        }
+    )
+
+    assert (
+        lhs.join_asof(rhs, on="a", by=["by", "by2"], strategy="backward")
+        .select(["a", "by"])
+        .frame_equal(pl.DataFrame({"a": [-20, -19, 8, 12, 14], "by": [1, 1, 2, 2, 2]}))
+    )
