@@ -1,3 +1,6 @@
+import pandas as pd
+from test_series import verify_series_and_expr_api
+
 import polars as pl
 from polars import testing
 
@@ -139,3 +142,45 @@ def test_list_arr_empty() -> None:
         {"cars_first": [1, 2, 4, None], "cars_literal": [2, 1, 3, 3]}
     )
     assert out.frame_equal(expected)
+
+
+def test_list_argminmax() -> None:
+    s = pl.Series("a", [[1, 2], [3, 2, 1]])
+    expected = pl.Series("a", [0, 2], dtype=pl.UInt32)
+    verify_series_and_expr_api(s, expected, "arr.arg_min")
+    expected = pl.Series("a", [1, 0], dtype=pl.UInt32)
+    verify_series_and_expr_api(s, expected, "arr.arg_max")
+
+
+def test_list_shift() -> None:
+    s = pl.Series("a", [[1, 2], [3, 2, 1]])
+    expected = pl.Series("a", [[None, 1], [None, 3, 2]])
+    assert s.arr.shift().to_list() == expected.to_list()
+
+
+def test_list_diff() -> None:
+    s = pl.Series("a", [[1, 2], [10, 2, 1]])
+    expected = pl.Series("a", [[None, 1], [None, -8, -1]])
+    assert s.arr.diff().to_list() == expected.to_list()
+
+
+def test_slice() -> None:
+    vals = [[1, 2, 3, 4], [10, 2, 1]]
+    s = pl.Series("a", vals)
+    assert s.arr.head(2).to_list() == [[1, 2], [10, 2]]
+    assert s.arr.tail(2).to_list() == [[3, 4], [2, 1]]
+    assert s.arr.tail(200).to_list() == vals
+    assert s.arr.head(200).to_list() == vals
+    assert s.arr.slice(1, 2).to_list() == [[2, 3], [2, 1]]
+
+
+def test_cast_inner() -> None:
+    a = pl.Series([[1, 2]])
+    for t in [bool, pl.Boolean]:
+        b = a.cast(pl.List(t))
+        assert b.dtype == pl.List(pl.Boolean)
+        assert b.to_list() == [[True, True]]
+
+    # this creates an inner null type
+    df = pl.from_pandas(pd.DataFrame(data=[[[]], [[]]], columns=["A"]))
+    assert df["A"].cast(pl.List(int)).dtype.inner == pl.Int64  # type: ignore
